@@ -1,6 +1,6 @@
 package PDF::API2::Resource::BaseFont;
 
-our $VERSION = '2.020'; # VERSION
+our $VERSION = '2.025'; # VERSION
 
 use base 'PDF::API2::Resource';
 
@@ -13,7 +13,7 @@ no warnings qw[ deprecated recursion uninitialized ];
 
 =head1 NAME
 
-PDF::API2::Resource::BaseFont
+PDF::API2::Resource::BaseFont - Base class for font resources
 
 =head1 METHODS
 
@@ -118,7 +118,7 @@ sub tounicodemap {
     $cmap.=qq|   /Supplement 0\n|;
     $cmap.=qq|>> def\n|;
     $cmap.=sprintf(qq|/CMapName /pdfapi2-%s+0 def\n|,$self->name);
-    if(UNIVERSAL::can($self,'uniByCId') && UNIVERSAL::can($self,'glyphNum')) {
+    if($self->can('uniByCId') and $self->can('glyphNum')) {
         # this is a type0 font
         $cmap.=sprintf(qq|1 begincodespacerange <0000> <%04X> endcodespacerange\n|,$self->glyphNum-1);
         for(my $j=0;$j<$self->glyphNum;$j++) {
@@ -624,14 +624,18 @@ is used either in native or utf8 format (check utf8-flag).
 sub width {
     my ($self,$text)=@_;
     my $width=0;
+    my @widths_cache;
     if(is_utf8($text)) {
         $text=$self->strByUtf($text)
     }
+
+    my $kern = $self->{-dokern} && ref($self->data->{kern});
     my $lastglyph='';
     foreach my $n (unpack('C*',$text)) 
     {
-        $width+=$self->wxByEnc($n);
-        if($self->{-dokern} && ref($self->data->{kern}))
+        $widths_cache[$n] = $self->wxByEnc($n) unless defined $widths_cache[$n];
+        $width += $widths_cache[$n];
+        if ($kern)
         {
             $width+=$self->data->{kern}->{$lastglyph.':'.$self->data->{e2n}->[$n]};
             $lastglyph=$self->data->{e2n}->[$n];
@@ -786,14 +790,8 @@ sub text
 
 sub isvirtual { return(0); }
 
-1;
-
-__END__
-
 =back
 
-=head1 AUTHOR
-
-Alfred Reibenschuh.
-
 =cut
+
+1;
